-- Includes
require('Code/universal.lua')
require('Code/ship.lua')
require('Code/enemy.lua')
require('Code/effect.lua')

--function love.load()
   -- Background
   background = love.graphics.newImage( 'Images/background.jpg' )
   love.audio.stop(m_menu)
   -- HUD
   start_timer = 4
   if score ~= 0 then
      score = 0
   end
   score_multiplier = 2
   target = love.graphics.newImage( 'Images/target.png' )
   hud = love.graphics.newFont(30)
   -- Ships
   health = 20
   ship = Ship:new(true, health, love.physics.newBody( world, screen_width/2, screen_height/2, 5, 0 ),
                   15, 10, 0, 2, 0, 'yellow',
                   5, 50, false, 0, 50/4, true)
   enemy_max = 4
   enemies = {}
   -- Effects
   particle = love.graphics.newImage( 'Images/particle.png' )
   ship_fire = Effect:new(particle, 500, 300, 400, 0.5, 0.1, 0.1, 0.2, 360, -3000)
   ship_fire:new_effect(particle, 500, 0, 'white', 0, 0)
   fire = Effect:new(particle, 500, 300, 400, 0.5, 0.1, 0.1, 0.2, 360, -3000)
   ship_explosion = Effect:new(particle, 100, 100, 200, 1, 1, 1, 1, 360, 0)
   explosion = Effect:new(particle, 100, 100, 200, 1, 1, 1, 1, 360, 0)
   explosions = {}
   exp_ref = 0
--end

function love.update(dt)
   if start_timer > 0 then
      start_timer = start_timer - dt
   end
   if start_timer < 1 then
       -- Updates
       world:update(dt)
       ship:update(dt, love.mouse.getX(), love.mouse.getY())
       for index, enemy in ipairs(enemies) do
          enemy:update(dt, health, ship.body:getX(), ship.body:getY(), ship.active)
       end
       if ship.active then
           ship_fire:get_effect():start()
           ship_fire:set_emission((health - ship.health) * 25)
           ship_fire:set_colour(ship.colour)
           ship_fire:set_position(ship.body:getX(), ship.body:getY())
           ship_fire:get_effect():update(dt)
       end
       for index, exp in ipairs(explosions) do
          exp:update(dt)
       end
       ship_explosion:get_effect():update(dt)
       -- Enemy spawns
       if table.getn(enemies) < enemy_max then
          local x, y = random_position()
          local d, s, c, bs, br = difficulty()
          fire:new_effect(particle, 500, 0, c, 0, 0)
          table.insert( enemies, Enemy:new(health, d, fire:get_effect(), love.physics.newBody( world, x, y, 5, 0 ),
                        15, s, 10, c,
                        5, bs, false, 0, br, false) )
       end
       -- Collisions
       for index, enemy in ipairs(enemies) do
          if (collision(ship.body:getX(), ship.body:getY(), ship.radius,
                       enemy.body:getX(), enemy.body:getY(), enemy.radius) and ship.active) then
             local x, y = separation(enemy.body:getX(), enemy.body:getY(), enemy.radius,
                                     ship.body:getX(), ship.body:getY())         
             enemy.body:setX(x)
             enemy.body:setY(y)
          end
       end
       for index1, enemy1 in ipairs(enemies) do
          for index2, enemy2 in ipairs(enemies) do
             if enemy1 ~= enemy2 then
                if collision(enemy1.body:getX(), enemy1.body:getY(), enemy1.radius,
                             enemy2.body:getX(), enemy2.body:getY(), enemy2.radius) then
                   local x, y = separation(enemy1.body:getX(), enemy1.body:getY(), enemy1.radius,
                                           enemy2.body:getX(), enemy2.body:getY())
                   enemy1.body:setX(x)
                   enemy1.body:setY(y)
                end
             end
          end
       end
       for index1, enemy in ipairs(enemies) do
          for index2, bullet in ipairs(ship.bullets) do
             if collision(enemy.body:getX(), enemy.body:getY(), enemy.radius,
                          bullet:getX(), bullet:getY(), ship.bullet.radius) then
                table.remove( ship.bullets, index2 )
                enemy.health = enemy.health - enemy.damage
                score = score + ((health-enemy.health) * score_multiplier)
                play_sound(s_damage)
                if enemy.health <= 0 then
                   explosion:new_effect(particle, 100, 500, enemy.colour,
                                        enemy.body:getX(), enemy.body:getY())
                   table.insert( explosions, explosion:get_effect() )
                   exp_ref = exp_ref + 1
                   explosions[exp_ref]:start()
                   table.remove( enemies, index1 )
                   play_sound(s_explosion)
                end
             end
          end
       end
       for index1, enemy in ipairs(enemies) do
          for index2, bullet in ipairs(enemy.bullets) do
             if (collision(ship.body:getX(), ship.body:getY(), ship.radius,
                           bullet:getX(), bullet:getY(), enemy.bullet.radius) and ship.active) then
                table.remove( enemy.bullets, index2 )
                ship.health = ship.health - 2
                play_sound(s_damage)
                if (ship.health <= 0 and ship.active) then
                   ship.active = false
                   if score > highscore then
                      highscore = score
                      love.filesystem.write('highscore.txt', score)
                   end
                   ship_fire:get_effect():reset()
                   ship_explosion:new_effect(particle, 100, 500, ship.colour,
                                        ship.body:getX(), ship.body:getY())
                   ship_explosion:get_effect():start()
                   play_sound(s_explosion)
                end
             end
          end
       end
       -- Explosion ends
       if ship_explosion:get_effect():isActive() == false then
          love.filesystem.load('Code/End.lua')()
       end
       for index, exp in ipairs(explosions) do
          if exp:isActive() == false then
             table.remove( explosions, index )
             exp_ref = exp_ref - 1
          end
       end
   end    
end

function love.draw()
   -- Background
   love.graphics.draw( background, 0, 0 )
   -- Effects
   love.graphics.draw(ship_fire:get_effect())
   love.graphics.draw(ship_explosion:get_effect())
   for index, exp in ipairs(explosions) do
      love.graphics.draw(exp)
   end
   -- Ships
   ship:draw()
   for index, enemy in ipairs(enemies) do
      enemy:draw()
   end
   -- HUD
   colour('white')
   love.graphics.setFont(hud)
   if (start_timer <= 4 and start_timer > 3) then
      love.graphics.print('3', (screen_width/2) - 10, (screen_height/2) - 50)
   elseif (start_timer <= 3 and start_timer > 2) then
      love.graphics.print('2', (screen_width/2) - 10, (screen_height/2) - 50)
   elseif (start_timer <= 2 and start_timer > 1) then
      love.graphics.print('1', (screen_width/2) - 10, (screen_height/2) - 50)
   elseif (start_timer <= 1 and start_timer > 0) then
      love.graphics.print('Go', (screen_width/2) - 20, (screen_height/2) - 50)
   end
   if ship.active then
       love.graphics.draw( target, love.mouse.getX() - (target:getWidth()/2),
                           love.mouse.getY() - (target:getHeight()/2) )
   end
   love.graphics.print('Score:', 50, 50)
   love.graphics.print(score, 150, 50)
   love.graphics.print('Highscore:', 50, screen_height - 50)
   if score > highscore then
      love.graphics.print(score, 220, screen_height - 50)
   else
      love.graphics.print(highscore, 220, screen_height - 50)
   end
end

function love.keypressed(key)
   if key == 'escape' then
      love.filesystem.load('Code/Start.lua')()
   end
end