-- Includes
require('Code/universal.lua')
require('Code/bullet.lua')

Ship = {} -- Create a table to hold the class methods

-- The constructor
function Ship:new(a, h, b, r, sm, s, ac, t, c, -- Ship itself
                  rs, sd, st, te, rt, it) -- Ship bullet
  local object = { active = a, health = h, body = b, radius = r, speed_max = sm, speed_left = s,
                   speed_right = s, speed_up = s, speed_down = s, acc = ac, turn_speed = t, colour = c,
                   bullet = Bullet:new(rs, sd, st, te, rt, it), bullets = {} }
  setmetatable(object, { __index = Ship }) -- Inheritance
  return object
end

function Ship:update(dt, tar_x, tar_y)
   if self.active then
       if screen_edge( self.body:getX(), self.body:getY(), self.radius ) == 1 then
          self.body:setX( 0+self.radius )
          self.speed_left = 0
       elseif screen_edge( self.body:getX(), self.body:getY(), self.radius ) == 2 then
          self.body:setX( screen_width-self.radius )
          self.speed_right = 0
       end
       if screen_edge( self.body:getX(), self.body:getY(), self.radius ) == 3 then
          self.body:setY( 0+self.radius )
          self.speed_up = 0
       elseif screen_edge( self.body:getX(), self.body:getY(), self.radius ) == 4 then
          self.body:setY( screen_height-self.radius )
          self.speed_down = 0
       end
       
       self:ship_force(dt)
       self:ship_gaze(tar_x, tar_y)
       self.bullet:birth(self.body, self.bullets, dt)
       self.bullet:death(self.bullets)
   end
end

function Ship:ship_force(dt)
   self.speed_left = self:force_direction( dt, 'left', 'a', self.speed_left )
   self.body:setX( self.body:getX() - self.speed_left )

   self.speed_right = self:force_direction( dt, 'right', 'd', self.speed_right )
   self.body:setX( self.body:getX() + self.speed_right )

   self.speed_up = self:force_direction( dt, 'up', 'w', self.speed_up )
   self.body:setY( self.body:getY() - self.speed_up )

   self.speed_down = self:force_direction( dt, 'down', 's', self.speed_down )
   self.body:setY( self.body:getY() + self.speed_down )
end

function Ship:force_direction( dt, key1, key2, dir )
   if ( love.keyboard.isDown(key1) or love.keyboard.isDown(key2) ) then
      if dir < self.speed_max then
         dir = dir + self.acc * (dt*5)
      end
         return dir
   elseif ( love.keyboard.isDown(key1) == false or love.keyboard.isDown(key2) == false ) then
      if dir > 0 then
         dir = dir - self.acc * (dt*5)
         return dir
      end
   end
   return 0
end

function Ship:ship_gaze(tar_x, tar_y)
   self.body:setAngle( -turn_to_face( self.body:getX(), self.body:getY(), tar_x, tar_y,
                       self.body:getAngle(), self.turn_speed ) - (math.pi/2) )
end

function Ship:draw()
   if self.active then
       colour(self.colour)
       draw_ship( self.body:getX(), self.body:getY(), self.body:getAngle(), self.radius )
       self.bullet:draw(self.bullets)
       love.graphics.rectangle( 'fill', self.body:getX() + self.radius, self.body:getY() - self.radius,
                                self.health, 5 )
   end
end