-- Global variables and functions

screen_width = 1280
screen_height = 720
world = love.physics.newWorld( screen_width, screen_height )
world:setGravity( 0, 0 )
score = 0
highscore = 0 + love.filesystem.read('highscore.txt')
m_menu = love.audio.newSource('Audio/menu.wav')
m_menu:setVolume(3.0)
m_menu:setLooping(true)
s_shoot = love.audio.newSource('Audio/shoot.wav', 'static')
s_shoot:setVolume(0.2)
s_damage = love.audio.newSource('Audio/damage.wav', 'static')
s_explosion = love.audio.newSource('Audio/explosion.wav', 'static')
s_explosion:setVolume(0.5)

function play_sound(sound)
   love.audio.stop(sound)
   love.audio.play(sound)
end

function colour(string)
   if string == 'red' then
      love.graphics.setColor( 255, 0, 0, 255 )
   elseif string == 'green' then
      love.graphics.setColor( 0, 255, 0, 255 )
   elseif string == 'blue' then
      love.graphics.setColor( 0, 0, 255, 255 )
   elseif string == 'yellow' then
      love.graphics.setColor( 255, 255, 0, 255 )
   else
      love.graphics.setColor( 255, 255, 255, 255 )
   end
end

function draw_ship(x, y, a, r)
   love.graphics.setLine(10)

   local p1x = (x-math.sin(a)*r)
   local p1y = (y-math.cos(a)*r)
   local p2x = (x+math.sin(a)*r/2)
   local p2y = (y+math.cos(a)*r/2)
   local p3x = (x+math.sin(a-0.8)*r)
   local p3y = (y+math.cos(a-0.8)*r)
   local p4x = (x+math.sin(a+0.8)*r)
   local p4y = (y+math.cos(a+0.8)*r)

   love.graphics.line(p1x,p1y, p4x,p4y)
   love.graphics.line(p1x,p1y, p3x,p3y)
   love.graphics.line(p4x,p4y, p2x,p2y)
   love.graphics.line(p3x,p3y, p2x,p2y)
end

function random_position()
   local result = math.random(1, 4)
   local x, y = 0
   
   if result == 1 then
      x = 0
      y = math.random(0, screen_height)
   elseif result == 2 then
      x = screen_width
      y = math.random(0, screen_height)
   elseif result == 3 then
      x = math.random(0, screen_width)
      y = 0
   elseif result == 4 then
      x = math.random(0, screen_width)
      y = screen_height
   end
      
   return x, y
end

function screen_edge( x, y, r )
   if x < 0+r then
      return 1
   elseif x > screen_width-r then
      return 2
   end
   if y < 0+r then
      return 3
   elseif y > screen_height-r then
      return 4
   end
   return 0
end

function difficulty()
   if (score >= 0 and score < 960) then
      return 10, 2, 'blue', 10, 50/2
   elseif (score >= 960 and score < 2880) then
      return 4, 4, 'green', 20, 50/2
   elseif score >= 2880 then
      return 2, 6, 'red', 40, 50/4
   end
end

function collision( x1, y1, r1, x2, y2, r2 )
   local distance = distance( x1, y1, r1, x2, y2, r2 )
   
   if distance <= 0 then
      return true
   end
   
   return false
end

function collision_point( x1, y1, x2, y2 )
   return ((x1+x2)/2), ((y1+y2)/2)
end

function distance( x1, y1, r1, x2, y2, r2 )
   local dif_x = x2 - x1
   local dif_y = y2 - y1

   local c_squared = math.pow(dif_x, 2) + math.pow(dif_y, 2)
   local distance = math.sqrt(c_squared) - r1 - r2
   
   return distance
end

function separation( x1, y1, r1, x2, y2 )
   local x = x1 - ((x2 - x1)/(r1-2))
   local y = y1 - ((y2 - y1)/(r1-2))
   
   return x, y
end

function turn_to_face( x, y, tar_x, tar_y, a, speed )
   local dif_x = tar_x - x
   local dif_y = tar_y - y

   local req_a = math.atan2( dif_y, dif_x )
   local dif = wrap_angle( req_a - a )
   if speed > 0 then
      dif = math.clamp(dif, -speed, speed)
   end

   return wrap_angle( a + dif )
end

function wrap_angle( rad )
   if rad < -math.pi then
      rad = rad + (math.pi*2)
   end
   if rad > math.pi then
      rad = rad - (math.pi*2)
   end
   return rad
end

function math.clamp(val, lower, upper)
    assert(val and lower and upper, 'one or more parameters are nil')
    if lower > upper then lower, upper = upper, lower end -- swap if boundaries supplied the wrong way
    return math.max(lower, math.min(upper, val))
end